from cgi import escape as cgi_escape


def escape(s, quote=None):
    if s is None:
        return ""
    return cgi_escape(s, quote)

def NA(text):
    if not text:
        return '-/-'
    return text

def nl_to_br(text):
    """Replace newlines in text with <br> tags"""
    text = text.replace('\r\n', '<br>')
    text = text.replace('\n', '<br>')
    text = text.replace('\r', '<br>')
    return text


def tag(tagname, _close=False,
        _booleans=('checked', 'disabled', 'multiple', 'selected'), **attrs):
    """Create an opening HTML tag with properly escaped attributes.

    The tagname parameter gives the name of the tag.  The attributes and
    their values are given as keyword parameters.  Most attributes have
    string values.  Boolean attributes are treated as described below.
    The values of string attributes are converted to strings if they are
    not aleady strings (byte-strings or unicode) using these rules: None
    is converted to the empty string, other values are converted using
    the str function.  After conversion the values of string attributes
    will be properly escaped as attribute values when inserted into the
    tag.

    Boolean flags are treated differently: if the value of such an
    attribute is true, the attribute is included in the tag without a
    value, if the value is false, the attribute is omitted.  The boolean
    attributes handled in this way are given by the _booleans parameter,
    which is a sequence of attribute names.  Its default value is
    ('checked', 'disabled', 'multiple', 'selected')

    To allow for attribute names that are also python keywords, a single
    trailing underscore character is removed from an attribute name if
    it is preset, so that e.g. the class attribute can be specified
    using the class_ keyword parameters.

    A few attribute names are not valid Python identifiers because they
    contain a minus-sign (e.g. accept-charset).  However, since there
    are no attribute names that contain underscores, underscores in a
    keyword argument will be converted to minus-signs to determine the
    attribute name.  For instance the accept-charset attribute can be
    given as accept_charset to this function.

    Examples:
    >>> from formed.renderer.helpers import tag
    >>> tag("input", type="text", name="title", disabled=False)
    '<input type="text" name="title">'
    >>> 
    """
    out = ["<%s" % tagname]
    for name, value in attrs.items():
        if name.endswith("_"):
            name = name[:-1]
        name = name.replace("_", "-")
        if name in _booleans:
            if value:
                out.append(' %s="%s"' % (name, name))
        else:
            if value is None:
                value = ""
            elif not isinstance(value, basestring):
                value = str(value)
            out.append(' %s="%s"' % (name, escape(value, True)))
    if _close:
        out.append(">")
    else:
        out.append(">")
    return "".join(out)
