#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2011 Intevation GmbH, Germany, <info@intevation.de>
# Authors:
# Björn Ricks <bjoern.ricks@intevation.de>
#
import sys
import logging

from optparse import OptionParser

from formed.meta.io.document import openDocument
from formed.export.sql import ExportAsSQL
from formed.export.rg_sql import ExportAsRepeatGroupSQLFilter
from formed.export.rules import ExportRulesForTagging, ExportRulesAsSH
from formed.export.xsd import ExportAsXMLSchema, ExportAsXMLSchemaExport
from formed.export.xls import ExportKeyValueTableAsXLS
from formed.export.required_rules import GenerateRequiredRules
from formed.export.clean import ExportCleanTree
from formed.export.dot import ExportAsDOT

FORMATS = {
    'sql':        ExportAsSQL,
    'rg_sql':     ExportAsRepeatGroupSQLFilter,
    'tagging':    ExportRulesForTagging,
    'tagging_sh': ExportRulesAsSH,
    'xsd':        ExportAsXMLSchema,
    'xsd_export': ExportAsXMLSchemaExport,
    'xls':        ExportKeyValueTableAsXLS,
    'rules':      GenerateRequiredRules,
    'clean':      ExportCleanTree,
    'dot':        ExportAsDOT,
    }

def main():
    parser = OptionParser(usage="usage: %prog [options] formedtree.xml "
                                "out.file")
    parser.add_option("-f", "--format", default="sql",
                      help="format to convert to")
    parser.add_option("-l", "--list-formats", dest="listformats",
                      action="store_true",
                      help="list formats and their description")
    options, args = parser.parse_args()

    logging.basicConfig(stream=sys.stderr)

    if options.listformats:
        print "Format\t\t\tDescription"
        for (format_name, format) in sorted(FORMATS.items()):
            print "%s\t\t\t%s" % (format_name, format.getDescription())
        return

    if len(args) < 1:
        print >>sys.stderr, "Error: Missing formedtree xml file"
        sys.exit(1)
    if len(args) < 2:
        print >>sys.stderr, "Error: Missing output file"
        sys.exit(2)

    if not options.format in FORMATS:
        print >> sys.stderr, "Error: Format %s is not supported. Supported " \
                             " formats are %s" % (options.format,
                                     FORMATS.keys())
        sys.exit(3)

    formed_file = args[0]
    out_file = args[1]
    format = FORMATS[options.format]

    formed = openDocument(formed_file)
    
    export = format(formed)
    export.doExport(out_file)


main()
