# Copyright (C) 2011 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>

import unittest

from formed.meta.exprtree import Expr, VariableProvider


class TestVariableProvider(VariableProvider):

    def __init__(self, variables):
        self.variables = variables

    def getValueForVar(self, var):
        return (var, self.variables[var])


class TestExpr(unittest.TestCase):

    def check_expr(self, expr, expected, **variables):
        self.assertEquals(Expr(expr).evaluate(TestVariableProvider(variables)),
                          expected)

    def test_contains_literals(self):
        self.check_expr("[ 4 8 12 ] 8 contains", True)
        self.check_expr("[ 4 9 16 25 ] 13 contains", False)

    def test_contains_expressions(self):
        self.check_expr("[ 4 2 * 12 ] 8 contains", True)
        self.check_expr("[ 4 6 8 + 15 ] 13 contains", False)

    def test_contains_variables(self):
        self.check_expr("[ 4 $var1 $var2 ] 8 contains", True, var1=5, var2=8)


if __name__ == "__main__":
    unittest.main()
