# -*- coding: utf-8 -*-
# Copyright (C) 2011 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>

import unittest
import filesupport
import StringIO

from formed.meta.io.document import openDocument
from formed.instance.backends.xmlimport import XMLImporter


class FormedTreeTest(filesupport.FileTestMixin):

    # overwrite in derived classes with the XML for the formed meta tree.
    formedtree = None

    def setUp(self):
        self.formedtree_filename = self.create_temp_file("formedtree.xml",
                                                         self.formedtree)
        self.meta_tree = openDocument(self.formedtree_filename)


class ImportTest(FormedTreeTest, unittest.TestCase):

    case_xml = None

    def setUp(self):
        FormedTreeTest.setUp(self)
        self.importer = XMLImporter(self.meta_tree)
        self.xml_infile = StringIO.StringIO(self.case_xml)

    def load_case(self):
        cases = list(self.importer.read_cases_from_xml(self.xml_infile))
        self.assertEquals(len(cases), 1)
        return cases[0]


class TestNumericImport(ImportTest):

    formedtree = """\
<?xml version="1.0" encoding="UTF-8"?>
<document>
  <case>
    <page containers="a" description="A Page" name="page-1">
      <float description="Float 1" name="float1" target="a"/>
    </page>
  </case>
  <logbook>
  </logbook>
</document>
"""

    case_xml = """\
<?xml version="1.0"?>
<cases formed-type="" version="5">
  <case>
    <master>
      <float1>10.0</float1>
    </master>
  </case>
</cases>
"""

    def test(self):
        case = self.load_case()
        master = case["master"]
        self.assertEquals(master.get("float1"), 10.0)
