#! /usr/bin/env python

"""Convert logbook settings from the web configuration for the offline client.
Usage:
  convert-logbook-settings.py webconfig.json offline/resources/logbook-meta.xml
"""

import sys
import xml.etree.ElementTree as ET

from mpulsweb.lib.config import MpulsAppConfig

def load_web_config(filename):
    return MpulsAppConfig(filename)


def create_entry_type(parent, value, text, editable=True):
    attrs = {"value": value}
    if not editable:
        attrs["editable"] = "false"
    entry_type = ET.SubElement(parent, "entry-type", attrs)
    entry_type.text = text
    return entry_type

def create_logbook_meta_element(web_config):
    logbook_meta = ET.Element("logbook-meta")
    logbook_meta.text = "\n  "
    logbook_meta.tail = "\n"

    entry_types = ET.SubElement(logbook_meta, "entry-types")
    entry_types.text = "\n    "
    entry_types.tail = "\n"

    # always add automatic entry which is not present in the json file
    entry_type = create_entry_type(entry_types, "0", "Automatischer Eintrag",
                                   editable=False)
    entry_type.tail = "\n    "

    descriptions = web_config.get('logbook', 'descriptions')[0]

    for num, category in enumerate(web_config.get('logbook', 'categories')):
        entry_category = ET.SubElement(entry_types, "entry-category",
                                       name=category["name"])
        entry_category.text = "\n      "
        entry_category.tail = "\n    "

        entry_type = None
        for item in category["items"]:
            if entry_type is not None:
                entry_type.tail = "\n      "
            entry_type = create_entry_type(entry_category, item,
                                           descriptions[item])

        if entry_type is not None:
            entry_type.tail = "\n    "

    entry_category.tail = "\n  "

    return logbook_meta

def write_logbook_meta(filename, logbook_meta):
    f = open(filename, "w")
    try:
        xml = ET.ElementTree(logbook_meta)
        f.write(ET.tostring(xml.getroot()))
    finally:
        f.close()


def main():
    if len(sys.argv) != 3:
        print __doc__
        exit(1)

    web_config_filename = sys.argv[1]
    logbook_meta_filename = sys.argv[2]

    web_config = load_web_config(web_config_filename)
    write_logbook_meta(logbook_meta_filename,
                       create_logbook_meta_element(web_config))

main()

