# -*- coding: utf-8 -*-
#
# Copyright (c) 2011 by Intevation GmbH
#
# FormEd is free software under the terms of the
# GNU General Public License Version 3. See file
# LICENSE coming with the source of FormEd for details.
#
# Authors:
# Bernhard Herzog <bernhard.herzog@intevation.de>
#

import re

from formed.meta.data import DateLeaf
from formed.export.required_rules import ORDER_RE


class ExportAsDOT:

    def __init__(self, document):
        self.document = document

    @staticmethod
    def getDescription():
        return "Export date order graph as dot file"

    def doExport(self, path):
        out = open(path, "w")
        try:
            self.write_graph(out)
        finally:
            out.close()

    def write_graph(self, out):
        out.write("digraph dateorder {\n")

        for date in self.document.findAllByClass(DateLeaf):
            name = date.getName()
            preds, succs = self.parse_order(date.getOrder())
            for pred in preds:
                self.write_edge(out, pred, name)
            for succ in succs:
                self.write_edge(out, name, succ)

        out.write("}\n")

    def write_edge(self, out, pred, succ):
        out.write("\t%s -> %s;\n" % (pred, succ))

    def parse_order(self, order):
        match = re.match(ORDER_RE, order)
        if match:
            preds = self.parse_order_field_list(match.group(1))
            succs = self.parse_order_field_list(match.group(2))
        else:
            preds = []
            succs = []
        return preds, succs

    def parse_order_field_list(self, fieldlist):
        fields = []
        for item in fieldlist.split(","):
            item = item.strip()
            if not item:
                continue
            # $TODAY is not a legal dot identifier, so use TODAY instead
            if item == "$TODAY":
                item = "TODAY"
            fields.append(item)
        return fields
