# -*- coding: utf-8 -*-
#
# Copyright (c) 2008 by Intevation GmbH
#
# FormEd is free software under the terms of the
# GNU General Public License Version 3. See file
# LICENSE coming with the source of FormEd for details.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
# Björn Ricks <bjoern.ricks@intevation.de>
#

import sys
import codecs
from cgi import escape
from time import gmtime, strftime

import formed.meta.data as data
from formed.meta.nodecomponents import Node, Leaf


XSD_HEADER = \
'''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">

<!-- XML schema generated by libformed
     %s -->

<xs:element name="document">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="case"    minOccurs="0" maxOccurs="unbounded"/>
            <xs:element ref="group"   minOccurs="0" maxOccurs="unbounded"/>
            <xs:element ref="page"    minOccurs="0" maxOccurs="unbounded"/>
            <xs:element ref="logbook" minOccurs="0" maxOccurs="1"/>
        </xs:choice>
        <xs:attribute name="version"       type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="choice">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="bool"  minOccurs="0" maxOccurs="unbounded"/>
            <xs:element ref="conditional"  minOccurs="0" maxOccurs="unbounded"/>
            <xs:element ref="group" minOccurs="0" maxOccurs="unbounded"/>
            <xs:element ref="external-choice" minOccurs="0" maxOccurs="unbounded"/>
        </xs:choice>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="invisible"   type="xs:string"/>
        <xs:attribute name="size"        type="xs:integer"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="flags"       type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="containers"  type="xs:string"/>
        <xs:attribute name="function"    type="xs:string"/>
        <xs:attribute name="modes"       type="xs:string"/>
        <xs:attribute name="formularname" type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="bool">
</xs:element>

<xs:element name="text">
</xs:element>

<xs:element name="page">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="group"    minOccurs="0"/>
            <xs:element ref="text"     minOccurs="0"/>
            <xs:element ref="date"     minOccurs="0"/>
            <xs:element ref="rule"     minOccurs="0"/>
            <xs:element ref="repeat"   minOccurs="0"/>
            <xs:element ref="textarea" minOccurs="0"/>
            <xs:element ref="choice"   minOccurs="0"/>
            <xs:element ref="int"      minOccurs="0"/>
            <xs:element ref="float"    minOccurs="0"/>
            <xs:element ref="conditional"  minOccurs="0"/>
            <xs:element ref="radio-matrix" minOccurs="0"/>
            <xs:element ref="info"     minOccurs="0"/>
            <xs:element ref="bool"     minOccurs="0"/>
        </xs:choice>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="containers"  type="xs:string"/>
        <xs:attribute name="invisible"   type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="formularname" type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="group">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="repeat"      minOccurs="0"/>
            <xs:element ref="choice"      minOccurs="0"/>
            <xs:element ref="page"        minOccurs="0"/>
            <xs:element ref="text"        minOccurs="0"/>
            <xs:element ref="rule"        minOccurs="0"/>
            <xs:element ref="date"        minOccurs="0"/>
            <xs:element ref="bool"        minOccurs="0"/>
            <xs:element ref="conditional" minOccurs="0"/>
            <xs:element ref="textarea"    minOccurs="0"/>
            <xs:element ref="group"       minOccurs="0"/>
            <xs:element ref="radio"       minOccurs="0"/>
            <xs:element ref="int"         minOccurs="0"/>
            <xs:element ref="float"       minOccurs="0"/>
            <xs:element ref="info"        minOccurs="0"/>
            <xs:element ref="radio-matrix" minOccurs="0"/>
        </xs:choice>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="invisible"   type="xs:string"/>
        <xs:attribute name="containers"  type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="flags"       type="xs:string"/>
        <xs:attribute name="modes"       type="xs:string"/>
        <xs:attribute name="repeat"      type="xs:string"/>
        <xs:attribute name="formularname" type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="logbook" type="xs:string"/>

<xs:element name="rule">
    <xs:complexType>
        <xs:attribute name="expr"        type="xs:string"/>
        <xs:attribute name="tag"         type="xs:string"/>
        <xs:attribute name="mark"        type="xs:string"/>
        <xs:attribute name="value"       type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="flags"       type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="int">
    <xs:complexType>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="minvalue"    type="xs:string"/>
        <xs:attribute name="maxvalue"    type="xs:string"/>
        <xs:attribute name="flags"       type="xs:string"/>
        <xs:attribute name="alternative" type="xs:string"/>
        <xs:attribute name="modes"       type="xs:string"/>
        <xs:attribute name="formularname" type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="float">
    <xs:complexType>
        <xs:attribute name="function"    type="xs:string"/>
        <xs:attribute name="maxvalue"    type="xs:string"/>
        <xs:attribute name="minvalue"    type="xs:string"/>
        <xs:attribute name="value"       type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="flags"       type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="info">
    <xs:complexType>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="value"       type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="external-choice">
    <xs:complexType>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="value"       type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="textarea">
    <xs:complexType>
        <xs:attribute name="function"    type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="rows"        type="xs:string"/>
        <xs:attribute name="cols"        type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="modes"       type="xs:string"/>
        <xs:attribute name="formularname" type="xs:string"/>
        <xs:attribute name="maxlength"   type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="radio-matrix">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="radio" minOccurs="0"/>
            <xs:element ref="rule"  minOccurs="0"/>
        </xs:choice>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="flags"       type="xs:string"/>
    </xs:complexType>
</xs:element>


<xs:element name="conditional">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="conditional" minOccurs="0"/>
            <xs:element ref="group"       minOccurs="0"/>
            <xs:element ref="choice"      minOccurs="0"/>
            <xs:element ref="textarea"    minOccurs="0"/>
            <xs:element ref="text"        minOccurs="0"/>
            <xs:element ref="date"        minOccurs="0"/>
            <xs:element ref="rule"        minOccurs="0"/>
            <xs:element ref="info"        minOccurs="0"/>
            <xs:element ref="int"         minOccurs="0"/>
            <xs:element ref="bool"        minOccurs="0"/>
            <xs:element ref="repeat"      minOccurs="0"/>
            <xs:element ref="radio-matrix" minOccurs="0"/>
        </xs:choice>
        <xs:attribute name="expr"        type="xs:string"/>
        <xs:attribute name="invisible"   type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="containers"  type="xs:string"/>
        <xs:attribute name="function"    type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="radio">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="bool" minOccurs="0"/>
            <xs:element ref="group"       minOccurs="0"/>
        </xs:choice>
        <xs:attribute name="description"  type="xs:string"/>
        <xs:attribute name="formularname" type="xs:string"/>
        <xs:attribute name="name"         type="xs:string"/>
        <xs:attribute name="size"         type="xs:string"/>
        <xs:attribute name="target"       type="xs:string"/>
        <xs:attribute name="flags"        type="xs:string"/>
        <xs:attribute name="selected"    type="xs:string"/>
        <xs:attribute name="modes"       type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:element name="repeat">
    <xs:complexType>
        <xs:choice maxOccurs="unbounded">
            <xs:element ref="text"     minOccurs="0"/>
            <xs:element ref="group"    minOccurs="0"/>
            <xs:element ref="textarea" minOccurs="0"/>
            <xs:element ref="int"      minOccurs="0"/>
            <xs:element ref="choice"   minOccurs="0"/>
            <xs:element ref="page"     minOccurs="0"/>
            <xs:element ref="date"     minOccurs="0"/>
            <xs:element ref="conditional"  minOccurs="0"/>
            <xs:element ref="radio-matrix" minOccurs="0"/>
        </xs:choice>
        <xs:attribute name="containers"  type="xs:string"/>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="min"         type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="digest"      type="xs:string"/>
        <xs:attribute name="orderby"     type="xs:string"/>
        <xs:attribute name="reverse"     type="xs:string"/>
    </xs:complexType>
</xs:element>


<xs:element name="date">
    <xs:complexType>
        <xs:attribute name="description" type="xs:string"/>
        <xs:attribute name="flags"       type="xs:string"/>
        <xs:attribute name="function"    type="xs:string"/>
        <xs:attribute name="name"        type="xs:string"/>
        <xs:attribute name="order"       type="xs:string"/>
        <xs:attribute name="target"      type="xs:string"/>
        <xs:attribute name="modes"       type="xs:string"/>
        <xs:attribute name="formularname" type="xs:string"/>
    </xs:complexType>
</xs:element>

<xs:simpleType name="version_type">
  <xs:restriction base="xs:string">
    <xs:pattern value="[0-5]{1}"/>
  </xs:restriction>
</xs:simpleType>

<xs:element name="cases">
  <xs:complexType>
    <xs:sequence>
        <xs:element ref="case" minOccurs="0" maxOccurs="unbounded"/>
    </xs:sequence>
    <xs:attribute name="version" type="version_type"/>
  </xs:complexType>
</xs:element>

<xs:simpleType name="tagebuch-datum_type">
  <xs:restriction base="xs:dateTime">
    <xs:pattern value="[0-9]{1,4}-[0-9]{1,2}-[0-9]{1,2}T[0-9]{2}:[0-9]{2}:[0-9]{2}"/>
  </xs:restriction>
</xs:simpleType>

<xs:element name="tagebuch-eintrag">
  <xs:complexType>
    <xs:sequence>
      <xs:element minOccurs="0" maxOccurs="1" name="uuid_id" type="uuid_type"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-datum" type="tagebuch-datum_type"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-bearbeiter" type="xs:string"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-art" type="xs:integer"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-sonstiges" type="xs:string"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-notiz" type="xs:string"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-dauer" type="unknown_integer"/>
    </xs:sequence>
  </xs:complexType>
</xs:element>

<xs:element name="tagebuch">
  <xs:complexType>
    <xs:sequence>
      <xs:element ref="tagebuch-eintrag" minOccurs="0" maxOccurs="unbounded"/>
    </xs:sequence>
  </xs:complexType>
</xs:element>

<xs:simpleType name="uuid_type">
  <xs:restriction base="xs:string">
    <xs:pattern value="[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}"/>
  </xs:restriction>
</xs:simpleType>

<xs:simpleType name="simple_date_type">
  <xs:restriction base="xs:date">
    <xs:pattern value="[0-9]{1,4}-[0-9]{1,2}-[0-9]{1,2}"/>
  </xs:restriction>
</xs:simpleType>

<xs:simpleType name="unknown_integer">
  <xs:restriction base="xs:integer">
    <xs:minInclusive value="-999999"/>
    <xs:maxInclusive value="999999"/>
  </xs:restriction>
</xs:simpleType>

<xs:element name="repeat_group_type" abstract="true"/>

<xs:element name="case">
  <xs:complexType>
    <xs:choice maxOccurs="unbounded">
        <xs:element ref="group"             minOccurs="0"/>
        <xs:element ref="repeat_group_type" minOccurs="0"/>
        <xs:element ref="tagebuch"          minOccurs="0"/>
        <xs:element ref="page"              minOccurs="0"/>
        <xs:element ref="repeat"            minOccurs="0"/>
    </xs:choice>
  </xs:complexType>
</xs:element>

'''


XSD_HEADER_EXPORT = \
'''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">

<xs:element name="cases">
  <xs:complexType>
    <xs:sequence>
      <xs:element ref="case" minOccurs="0" maxOccurs="unbounded"/>
    </xs:sequence>
    <xs:attribute name="version" type="version_type"/>
    <xs:attribute name="formed-type" type="xs:string"/>
  </xs:complexType>
</xs:element>

<xs:element name="case">
  <xs:complexType>
    <xs:sequence>
        <xs:element ref="master" minOccurs="0" maxOccurs="1"/>
        <xs:element ref="repeat_group_type" minOccurs="0" maxOccurs="unbounded"/>
        <xs:element ref="tagebuch" minOccurs="0" maxOccurs="1"/>
    </xs:sequence>
  </xs:complexType>
</xs:element>

<xs:simpleType name="version_type">
  <xs:restriction base="xs:string">
    <xs:pattern value="[0-5]{1}"/>
  </xs:restriction>
</xs:simpleType>

<xs:element name="repeat_group_type" abstract="true"/>

<xs:simpleType name="uuid_type">
  <xs:restriction base="xs:string">
    <xs:pattern value="[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}"/>
  </xs:restriction>
</xs:simpleType>

<xs:simpleType name="simple_date_type">
  <xs:restriction base="xs:date">
    <xs:pattern value="[0-9]{1,4}-[0-9]{1,2}-[0-9]{1,2}"/>
  </xs:restriction>
</xs:simpleType>

<xs:simpleType name="unknown_integer">
  <xs:restriction base="xs:integer">
    <xs:minInclusive value="-999999"/>
    <xs:maxInclusive value="999999"/>
  </xs:restriction>
</xs:simpleType>

<xs:simpleType name="tagebuch-datum_type">
  <xs:restriction base="xs:dateTime">
    <xs:pattern value="[0-9]{1,4}-[0-9]{1,2}-[0-9]{1,2}T[0-9]{2}:[0-9]{2}:[0-9]{2}"/>
  </xs:restriction>
</xs:simpleType>

<xs:element name="tagebuch-eintrag">
  <xs:complexType>
    <xs:sequence>
      <xs:element minOccurs="0" maxOccurs="1" name="uuid_id" type="uuid_type"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-datum" type="tagebuch-datum_type"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-bearbeiter" type="xs:string"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-art" type="xs:integer"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-sonstiges" type="xs:string"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-notiz" type="xs:string"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-dauer" type="unknown_integer"/>
      <xs:element minOccurs="0" maxOccurs="1" name="tagebuch-typ" type="xs:integer"/>
    </xs:sequence>
  </xs:complexType>
</xs:element>

<xs:element name="tagebuch">
  <xs:complexType>
    <xs:sequence>
      <xs:element ref="tagebuch-eintrag" minOccurs="0" maxOccurs="unbounded"/>
    </xs:sequence>
  </xs:complexType>
</xs:element>




<!-- XML schema generated by libformed
     %s -->
'''



XSD_FOOTER = \
'''</xs:schema>
'''

ANNOTATION = \
'''  <xs:annotation>
    <xs:documentation xml:lang="de">
      <ul xmlns="http://www.w3.org/1999/xhtml">
        %s
      </ul>
    </xs:documentation>
  </xs:annotation>'''


class Type:

    def __init__(self, name, nc):
        self.name = name
        self.nc = nc

    def buildDescriptionChain(self):
        n = self.nc
        descs = []
        while n:
            desc = n.getDescription()
            if desc:
                if descs and descs[0] == desc:
                    n = n.getParent()
                    continue
                descs.insert(0, desc)
            n = n.getParent()
        return descs

    def dumpAnnotation(self, out):
        chain = self.buildDescriptionChain()
        if not chain:
            return
        chain = "\n        ".join([u"<li>%s</li>" % escape(d) for d in chain])
        print >> out, ANNOTATION % chain

    def dump(self, out):
        pass


class BasedType(Type):

    def __init__(self, name, nc, base):
        Type.__init__(self, name, nc)
        self.base = base

    def dump(self, out):
        print >> out, '<xs:simpleType name="%s_type">' % self.name
        self.dumpAnnotation(out)
        print >> out, '  <xs:restriction base="%s"/>' % self.base
        print >> out, '</xs:simpleType>'


class TextAreaType(Type):

    def __init__(self, name, nc):
        Type.__init__(self, name, nc)

    def dump(self, out):
        print >> out, '<xs:simpleType name="%s_type">' % self.name
        self.dumpAnnotation(out)
        print >> out, '  <xs:restriction base="xs:string"/>'
        print >> out, '</xs:simpleType>'


class TextType(Type):

    def __init__(self, name, nc):
        Type.__init__(self, name, nc)

    def dump(self, out):
        print >> out, '<xs:simpleType name="%s_type">' % self.name
        self.dumpAnnotation(out)
        maxLength = self.nc.getMaxLength()

        if maxLength != "":
            maxLength = int(maxLength)
        else:
            maxLength = None

        if maxLength is None:
            print >> out, '  <xs:restriction base="xs:string"/>'
        else:
            print >> out, '  <xs:restriction base="xs:string">'
            print >> out, '    <xs:maxLength value="%d"/>' % maxLength
            print >> out, '  </xs:restriction>'
        print >> out, '</xs:simpleType>'


class DateType(Type):

    def __init__(self, name, nc):
        Type.__init__(self, name, nc)

    def dump(self, out):
        print >> out, '<xs:simpleType name="%s_type">' % self.name
        self.dumpAnnotation(out)
        print >> out, '  <xs:restriction base="simple_date_type"/>'
        print >> out, '</xs:simpleType>'


class IntegerType(Type):

    def __init__(self, name, nc):
        Type.__init__(self, name, nc)

    def dump(self, out):
        print >> out, '<xs:simpleType name="%s_type">' % self.name
        self.dumpAnnotation(out)
        print >> out, '  <xs:union memberTypes="unknown_integer">'
        print >> out, '    <xs:simpleType>'
        print >> out, '      <xs:restriction base="xs:integer">'

        minV, maxV = self.nc.getMinValue(), self.nc.getMaxValue()

        if not minV is None and not maxV is None:
            minV, maxV = min(minV, maxV), max(minV, maxV)

        if not minV is None:
            print >> out, '        <xs:minInclusive value="%d"/>' % minV

        if not maxV is None:
            print >> out, '        <xs:maxInclusive value="%d"/>' % maxV

        print >> out, '      </xs:restriction>'
        print >> out, '    </xs:simpleType>'
        print >> out, '  </xs:union>'
        print >> out, '</xs:simpleType>'


class EnumerationType:

    def __init__(self, name, keys_values):
        self.name = name
        self.keys_values = keys_values

    def isSame(self, other):
        return self.keys_values == other.keys_values

    def dump(self, out):
        print >> out, '<xs:simpleType name="%s">' % self.name
        print >> out, '  <xs:annotation>'
        print >> out, '    <xs:documentation xml:lang="de">'
        print >> out, '      <table xmlns="http://www.w3.org/1999/xhtml">'
        print >> out, u'        <tr><th>Schlüssel</th><th>Wert</th></tr>'
        for k, v in self.keys_values:
            print >> out, (u'        <tr><td>%d</td><td>%s</td></tr>'
                           % (k, escape(v)))
        print >> out, '      </table>'
        print >> out, '    </xs:documentation>'
        print >> out, '  </xs:annotation>'
        print >> out, '  <xs:restriction base="xs:integer">'
        for kv in self.keys_values:
            print >> out, '    <xs:enumeration value="%d"/>' % kv[0]
        print >> out, '  </xs:restriction>'
        print >> out, '</xs:simpleType>'


class TableType:

    def __init__(self, name, maxOccurs="unbounded"):
        self.name = name
        self.items = []
        self.repeat_groups = []
        self.maxOccurs = maxOccurs

    def dump(self, out):
        subst = self.name != "master" and ' substitutionGroup="repeat_group_type"' or ''
        print >> out, '<xs:element name="%s"%s>' % (self.name, subst)
        print >> out, '  <xs:complexType>'
        print >> out, '    <xs:sequence>'
        print >> out, '      <xs:element minOccurs="0" maxOccurs="1" name="uuid_id" type="uuid_type"/>'
        for item in self.items:
            print >> out, '      <xs:element minOccurs="0" maxOccurs="1" name="%s" type="%s_type"/>' % (
                item.name, item.name)
        for repeat_group in self.repeat_groups:
            print >> out, '      <xs:element minOccurs="0" maxOccurs="unbounded" name="%s"/>' % (repeat_group.name)
        print >> out, '    </xs:sequence>'
        print >> out, '  </xs:complexType>'
        print >> out, '</xs:element>'

    def dumpTypes(self, out):
        for item in self.items:
            item.dump(out)

    def appendItem(self, item):
        self.items.append(item)

    def appendRepeatGroup(self, rg):
        self.repeat_groups.append(rg)


class CaseType:

    def __init__(self):
        self.tables = []

    def dumpTables(self, out):
        for table in self.tables:
            table.dump(out)
            table.dumpTypes(out)

    def appendTable(self, table):
        self.tables.append(table)


class SchemaCreator:

    def __init__(self):
        pass

    def createSchema(self, root):
       self.enums = []
       case = CaseType()
       master = TableType("master", "1")
       case.appendTable(master)
       self.case = case
       self.tableStack = [master]
       self._createRecursive(root)

    def getCase(self):
        return self.case

    def findEnumeration(self, enum):
        for e in self.enums:
            if e.isSame(enum):
                return e
        return None

    def dumpEnumerations(self, out):
        for e in self.enums:
            e.dump(out)

    def _createRecursive(self, node):
        for child in node.children:
            self._createChild(child)

    def _createDate(self, date):
        self.tableStack[-1].appendItem(DateType(date.getName(), date))

    def _createBool(self, bool):
        key_values = [
            (-1, 'Keine Angabe'),
            (0, 'Nein'),
            (1, 'Ja')]

        enum_name = "enum_%d_type" % len(self.enums)

        enum = EnumerationType(enum_name, key_values)

        base = self.findEnumeration(enum)

        if not base:
            base = enum
            self.enums.append(enum)

        self.tableStack[-1].appendItem(BasedType(bool.getName(), bool,
                                                 base.name))

    def _createText(self, text):
        self.tableStack[-1].appendItem(TextType(text.getName(), text))

    def _createTextArea(self, text):
        self.tableStack[-1].appendItem(TextAreaType(text.getName(), text))

    def _createInteger(self, integer):
        self.tableStack[-1].appendItem(IntegerType(integer.getName(), integer))

    def _createChoice(self, choice):
        if choice.getMultiple():
            print >> sys.stderr, "WARNING: No support for multiple choices."
            return

        kvs = {}
        stack = choice.children[:]
        try:
            while stack:
                c = stack.pop()
                if isinstance(c, data.ExternalChoiceListLeaf):
                    cc = c.getChildren()
                    if cc:
                        stack.extend(cc)
                elif isinstance(c, data.GroupNode):
                    stack.extend(c.children)
                # Also add options if they are defined in a conditional
                elif isinstance(c, data.ConditionalNode):
                    stack.extend(c.children)
                elif isinstance(c, data.BoolLeaf):
                    desc = c.getDescription()
                    if not desc:
                        desc = ""
                    kvs[int(c.getValue())] = desc
        except ValueError:
            print >> sys.stderr, "WARNING choice '%s' contains no integer values" % name
            return

        if not kvs:
            print >> sys.stderr, "WARNING: choice '%s' contains no values" % choice.getName()
            return

        keys_values = list(kvs.iteritems())
        keys_values.sort()

        enum_name = "enum_%d_type" % len(self.enums)

        enum = EnumerationType(enum_name, keys_values)

        base = self.findEnumeration(enum)

        if not base:
            base = enum
            self.enums.append(enum)

        self.tableStack[-1].appendItem(BasedType(choice.getName(), choice,
                                                 base.name))

    def _createRadioGroup(self, radio):
        kvs = {}
        keys_values = []
        try:
            for c in radio.children:
                if isinstance(c, data.ExternalChoiceListLeaf):
                    cc = c.getChildren()
                    if cc:
                        for ccc in cc:
                            desc = ccc.getDescription()
                            if not desc:
                                desc = ""
                            kvs[int(ccc.getValue())] = desc
                elif isinstance(c, data.BoolLeaf):
                    desc = c.getDescription()
                    if not desc:
                        desc = ""
                    kvs[int(c.getValue())] = desc
        except ValueError:
            print >> sys.stderr, "WARNING: radio '%s' contains no integer values" % radio.getName()
            return

        if not kvs:
            print >> sys.stderr, "WARNING: radio '%s' contains no values" % radio.getName()
            return

        keys_values = list(kvs.iteritems())
        keys_values.sort()

        enum_name = "enum_%d_type" % len(self.enums)

        enum = EnumerationType(enum_name, keys_values)

        base = self.findEnumeration(enum)

        if not base:
            base = enum
            self.enums.append(enum)

        self.tableStack[-1].appendItem(BasedType(radio.getName(), radio,
                                                 base.name))

    def _createGroup(self, group):
        repeat = group.isRepeat()

        if repeat:
            table = TableType(group.getName())
            self.tableStack.append(table)
            self.tableStack[0].appendRepeatGroup(table)
            self.case.appendTable(table)

        for child in group.children:
            self._createChild(child)

        if repeat:
            self.tableStack.pop()

    def _createRepeat(self, repeat):
        table = TableType(repeat.getName())
        self.tableStack.append(table)
        self.tableStack[0].appendRepeatGroup(table)
        self.case.appendTable(table)

        for child in repeat.children:
            self._createChild(child)

        self.tableStack.pop()

    def _createChild(self, child):
        if isinstance(child, Leaf):
            if isinstance(child, data.IntLeaf):
                self._createInteger(child)
            elif isinstance(child, data.DateLeaf):
                self._createDate(child)
            elif isinstance(child, data.TextLeaf):
                self._createText(child)
            elif isinstance(child, data.TextAreaLeaf):
                self._createTextArea(child)
            elif isinstance(child, data.BoolLeaf):
                self._createBool(child)
        elif isinstance(child, Node):
            # not sure if GroupNode needs special handling
            # if not _createRecursive(child) should be enough
            # but it still has the attribute repeat
            if isinstance(child, data.GroupNode):
                self._createGroup(child)
            elif isinstance(child, data.RepeatNode):
                self._createRepeat(child)
            elif isinstance(child, data.RadioNode):
                self._createRadioGroup(child)
            elif isinstance(child, data.ChoiceNode):
                self._createChoice(child)
            else:
                self._createRecursive(child)


class ExportAsXMLSchema:

    def __init__(self, document):
        self.root = document.root

    @staticmethod
    def getDescription():
        return "Export document as XML schema"

    def doExport(self, path):
        root = self.root
        out = None
        try:
            out = codecs.open(path, "wb", "UTF-8")
            out.write(XSD_HEADER
                      % strftime("%a, %Y-%m-%d %H:%M:%S +0000", gmtime()))
            out.write(XSD_FOOTER)
        finally:
            if out:
                try:
                    out.close()
                except:
                    pass
        pass


class ExportAsXMLSchemaExport(ExportAsXMLSchema):

    @staticmethod
    def getDescription():
        return "Export document As XML Schema (export)"

    def doExport(self, path):
        out = None
        try:
            creator = SchemaCreator()
            creator.createSchema(self.root)
            out = codecs.open(path, "wb", "UTF-8")
            out.write(XSD_HEADER_EXPORT
                      % strftime("%a, %Y-%m-%d %H:%M:%S +0000", gmtime()))
            case = creator.getCase()
            case.dumpTables(out)
            creator.dumpEnumerations(out)
            out.write(XSD_FOOTER)
        finally:
            if out:
                try:
                    out.close()
                except:
                    pass
        pass

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
