# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import os.path

from codecs import lookup
from mimetypes import guess_type

IDENTITY = lambda x: x
__BASE64 = lookup('base64')[0]
BASE64 = lambda x: __BASE64(x)[0]

UTF8 = lambda x: unicode(x, 'utf-8')


class FileCache(object):

    def __init__(self, search_path):
        self.search_path = search_path
        self.cache = {}
        self.file_name_cache = {}

    def get_image(self, name, attributes=""):
        return u"".join([u'<img src="', self.get_image_url(name), u'" ',
                         attributes, u'>'])

    def get_image_url(self, name):
        ctype = guess_type(name)[0]
        if not ctype:
            ctype = "application/octet-stream"
        return u"".join(["data:", ctype, u';base64,',
                         self.get_file(name, BASE64)])

    def get_file_name(self, filename):
        full_name = self.file_name_cache.get(filename)
        if full_name is None:
            for directory in self.search_path:
                full_name = os.path.normpath(os.path.join(directory, filename))
                if os.path.exists(full_name):
                    break
            else:
                raise IOError("Could not find %r in search path %r"
                              % (filename, self.search_path))
            self.file_name_cache[filename] = full_name
        return full_name

    def get_file(self, filename, encoder=IDENTITY):
        filename = self.get_file_name(filename)
        try:
            entry = self.cache[filename]
        except KeyError:
            f = open(filename, "rb")
            try:
                content = f.read()
                entry = {'content': content}
                self.cache[filename] = entry
            finally:
                f.close()
        try:
            content = entry[encoder]
        except KeyError:
            content = encoder(entry['content'])
            entry[encoder] = content

        return content

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
