# -*- coding: utf-8 -*-
#
# (c) 2009 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import datetime
import re

from string import printable

PRINTABLE = frozenset(printable)

RE_HH_MM = re.compile("(\d{1,2})\s*:\s*(\d{1,2})")
RE_dd_mm_YYYY = re.compile("(\d{1,2})\s*\.\s*(\d{1,2})\s*\.\s*(\d{1,4})")
RE_pos_int = re.compile("(\d+)")

def iso_time_string(d):
    """datetime.strftime() dislikes years before 1900."""
    return "%04d-%02d-%02dT%02d:%02d:%02d" % (d.year, d.month, d.day,
                                              d.hour, d.minute, d.second)

def string_iso_time(s):
    return datetime.datetime.strptime(s.replace("-", ""), "%Y%m%dT%H:%M:%S")

def dd_mm_YYYY(d, empty=''):
    ''' datetime.strftime() dislikes years before 1900. '''
    if d is None:
        return empty
    return "%02d.%02d.%d" % (d.day, d.month, d.year)

def HH_MM(d, empty=''):
    ''' datetime.strftime() dislikes years before 1900. '''
    if d is None:
        return empty
    if type(d) is datetime.date:
        # not so smart
        d = datetime.datetime(d.year, d.month, d.day)
    return "%02d:%02d" % (d.hour, d.minute)

def parse_HH_MM(s):
    m = RE_HH_MM.search(s)
    if not m:
        raise ValueError(u'Keine gültige Uhrzeit')
    h, m = int(m.group(1)), int(m.group(2))
    if h > 23:
        raise ValueError(u'Keine gültige Stundenangabe')
    if m > 59:
        raise ValueError(u'Keine gültige Minutenangabe')
    return (h, m)

def parse_dd_mm_YYYY(s):
    m = RE_dd_mm_YYYY.search(s)
    if not m:
        raise ValueError(u'Kein gültiges Datum')
    d, m, y = int(m.group(1)), int(m.group(2)), int(m.group(3))
    if d > 31:
        raise ValueError(u'Kein gültiger Tag')
    if m > 12:
        raise ValueError(u'Kein gültiger Monat')
    if y < 80:
        y += 1900
    return (d, m, y)

def parse_pos_int(s):
    m = RE_pos_int.search(s)
    if not m:
        raise ValueError(u'Keine gültige Anzahl')
    return int(m.group(1))

def safe_unicode(s):
    return u''.join([c for c in s if ord(c) > 127 or c in PRINTABLE])

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
