# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

from xml.sax.handler import ContentHandler

from datetime import date

from helpers import iso_time_string, string_iso_time

import re

NO_NUMBER = re.compile("[^0-9\-]")
INT_NUMBER = re.compile("(-?[0-9]+)")
DATE = re.compile("([0-9]{1,4})-([0-9]{1,2})-([0-9]{1,2})")

def decode_int(s):
    s = NO_NUMBER.sub("", s)
    m = INT_NUMBER.match(s)
    if not m:
        return None
    return int(m.group(1))

def decode_date(s):
    s = NO_NUMBER.sub("", s)
    m = DATE.match(s)
    if not m:
        return None
    return date(int(m.group(1)), int(m.group(2)), int(m.group(3)))

def decode_iso_datetime(s):
    if not s:
        return None
    try:
        return string_iso_time(s)
    except:
        return None


def iso_datetime_string(d):
    return iso_time_string(d)


class StackedLoader(ContentHandler):

    def __init__(self):
        ContentHandler.__init__(self)
        self.factories = {}
        self.current = None
        self.depth = 0
        self.entry_levels = None
        self.stack = None

    def add_factories(self, tag, factories, default):
        self.factories[tag] = (factories, default)

    def activate_factories(self, tag):
        self.current = self.factories.get(tag)

    def startElement(self, name, attrs):
        new_factories = self.factories.get(name)

        if new_factories:
            self.entry_levels.append(self.depth)
            self.current = new_factories

        if self.current:
            start, end = self.current[0].get(name, self.current[1])
            start(name, attrs)
        else:
            end = None

        self.stack.append((end, attrs, []))

        self.depth += 1

    def endElement(self, name):
        t = self.stack.pop()

        if t[0]:
            t[0](name, t[1], u"".join(t[2]))

        self.depth -= 1
        if self.entry_levels and self.entry_levels[-1] == self.depth:
            self.entry_levels.pop()

    def activate_factories(self, tag):
        try:
            self.current = self.factories[tag]
        except KeyError:
            pass

    def characters(self, chars):
        self.stack[-1][2].append(chars)

    def startDocument(self):
        self.depth = 0
        self.entry_levels = []
        self.stack = []

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
