# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import sys
import os
import imp
from urllib import pathname2url
from optparse import OptionParser
import logging

from PyQt4.QtGui import QApplication

from formed.meta.io.document import openDocument
import formed.renderer.html

from lib.filecache import FileCache

from model.memory import MemoryDatastore
from model.defaults import Defaults
from model.logbook import LogbookMeta

from gui import MainWindow
from config import Config


def main_is_frozen():
    return (hasattr(sys, "frozen")    or # new py2exe
            hasattr(sys, "importers") or # old py2exe
            imp.is_frozen("__main__"))   # tools/freeze


def disable_logging():
    class _NullHandler(logging.Handler):

        def emit(self, record):
            pass

    logging.getLogger().addHandler(_NullHandler())



class Main(object):

    def __init__(self, app_dir, personal_defaults):
        parser = OptionParser()
        parser.add_option("-c", "--config", help="Path to contiguration file",
                          dest="config", default="offline.cfg")

        options, args = parser.parse_args()

        if main_is_frozen():
            disable_logging()
            base = os.path.dirname(sys.argv[0])
        else:
            logging.basicConfig()
            base = os.path.normpath(os.path.dirname(__file__))

        # make sure base is an absolute filename. This can happen in
        # some circumstances.
        base = os.path.abspath(base)

        self.config = Config(options.config)

        self.base_url = "local:/"

        search_path = [os.path.join(base, "resources")]
        if app_dir is not None:
            search_path.insert(0, app_dir)
        self.file_cache = FileCache(search_path)
        lookup = self.file_cache.get_file_name

        self.formed = openDocument(lookup(self.config.get_formed_path()))
        self.defaults = Defaults(lookup("formed/defaults.xml"),
                                 os.path.join(os.path.expanduser('~'), '.mpuls',
                                              personal_defaults))
        self.logbook_meta = LogbookMeta(lookup("formed/logbook-meta.xml"))

        if main_is_frozen():
            formed.renderer.html.i18n_base_dir = os.path.join(base, "i18n")

        self.document = None
        if args:
            self.load_document(args[0])

    def load_document(self, filename):
        swapfile = os.path.join(self.config.get_homedir(),
                                self.config.get_swapfile())
        self.document = MemoryDatastore(self.formed, self.defaults,
                                        self.logbook_meta, filename)
        self.document.setSwapfile(swapfile)

    def new_document(self):
        swapfile = os.path.join(self.config.get_homedir(),
                                self.config.get_swapfile())
        self.document = MemoryDatastore(self.formed, self.defaults,
                                        self.logbook_meta)
        self.document.setSwapfile(swapfile)



def main(app_dir=None, personal_defaults="defaults.xml"):
    app = QApplication(sys.argv)
    main = Main(app_dir=app_dir, personal_defaults=personal_defaults)
    main_window = MainWindow(main)
    main_window.show()
    sys.exit(app.exec_())

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
