# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import os
import re
import codecs
from datetime import date

from xml.sax import make_parser
from xml.sax.handler import ContentHandler
from xml.sax.saxutils import escape


VARIABLE = re.compile("\$\{([^\}]+)\}")

XML_HEADER = \
u'''<?xml version="1.0" encoding="UTF-8"?>
<defaults>'''

XML_FOOTER = u'</defaults>'


def extern(arg):
    stdout = os.popen(arg)
    result = stdout.read()
    stdout.close()
    return result.strip()

def today(arg):
    d = date.today()
    return u"%d-%d-%d" % (d.year, d.month, d.day)

SPECIALS = {
    'HEUTE': today,
    'EXTERN': extern
    }


class XMLHandler(ContentHandler):

    def __init__(self):
        self.values = []
        self.stack = []

    def startElement(self, name, attrs):
        if name == u"item":
            self.stack.append((attrs[u"key"], attrs.get(u"description"),
                               attrs.get(u"immutable", u"false") == u"true",
                               []))

    def characters(self, chars):
        if self.stack:
            self.stack[-1][-1].append(chars)

    def endElement(self, name):
        if name == u"item":
            i = self.stack.pop()
            self.values.append(Item(i[0], u"".join(i[3]), i[1], i[2]))


class Item(object):

    def __init__(self, key, value, description, immutable=False):
        self.key = key
        self.value = value
        self.description = description
        self.immutable = immutable

    def eval(self):
        def match(m):
            args = m.group(1).split(u':', 1)
            try:
                cmd = SPECIALS[args[0]]
            except KeyError:
                return u''
            if len(args) < 2:
                return cmd(u'')
            return cmd(args[1])

        return VARIABLE.sub(match, self.value)

    def write(self, out):
        print >> out, (u'<item key="%s"%s%s>%s</item>'
                       % (self.key,
                          self.description and (' description="%s"'
                                                % self.description) or '',
                          self.immutable and (' immutable="%s"'
                                              % self.immutable) or '',
                          escape(self.value)))


class Defaults(object):

    def __init__(self, global_defaults, personal_defaults):
        self.global_defaults = global_defaults
        self.personal_defaults = personal_defaults
        self.values = [] # use list to keep order
        self.read_defaults()

    def read_defaults(self):
        defaults_xml = self.personal_defaults

        if not os.path.isfile(defaults_xml):
            defaults_xml = self.global_defaults

        if os.path.isfile(defaults_xml):
            f = None
            try:
                xml_handler = XMLHandler()
                f = open(defaults_xml, "rb")
                parser = make_parser()
                parser.setContentHandler(xml_handler)
                parser.parse(f)
                self.values = xml_handler.values
            finally:
                if f:
                    f.close()

    def store(self):
        directory = os.path.dirname(self.personal_defaults)
        if not os.path.exists(directory):
            os.makedirs(directory)
        elif not os.path.isdir(directory):
            raise Exception("'%s' needs to be a directory" % directory)

        out = codecs.open(self.personal_defaults, "w", "UTF-8")
        try:
            print >> out, XML_HEADER
            for v in self.values:
                v.write(out)
            print >> out, XML_FOOTER
        finally:
            out.close()

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
