# -*- coding: utf-8 -*-
#
# (c) 2009 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

from string import Template
from cgi import escape

from offline.lib.filters import shorten
from offline.model.logbook import ORDER_ENTRIES_BY_DATE_DESC


OVERVIEW_TEMPLATE = Template(u'''
<div id="content_header_small">
  <div id="content_ueberschrift">
    Tagebuch
  </div>
  <div id="modusfield">
    <div class="actions">
      <ul>
        <li><a href="javascript:create_logbook_entry();">Eintrag vornehmen</a></li>
      </ul>
    </div>
  </div>
</div>
<table class="appointment">
  <tr>
    <th class="date table_header_h">Datum</th>
    <th class="title table_header_h">Bearbeiter</th>
    <th class="title table_header_h">Art</th>
    <th class="title table_header_h">Dauer (min)</th>
    <th class="action table_header_h">Aktionen</th>
  </tr>
  $OVERVIEW_ENTRIES
</table>
''')

ENTRY_TEMPLATE = Template(u'''
  <tr>
    <td class="date">$DATE $TIME</td>
    <td class="title">$EDITOR</td>
    <td class="title">$KIND</td>
    <td class="title">$DURATION</td>
    <td class="action">
        <a href="javascript:view_logbook_entry('$UUID');">$VIEW_IMAGE</a>
        $OPTIONAL_DELETE
    </td>
  </tr>
  <tr>
    <td colspan="5" class="description">$NOTICE</td>
  </tr>
''')

NO_ENTRIES = u'''
  <tr>
    <td colspan="5">Keine Eintr&auml;ge gefunden</td>
  </tr>
'''

VIEW_ENTRY_TEMPLATE = Template(u'''
<div id="content_header_small">
  <div id="content_ueberschrift">
    Tagebucheintrag
  </div>
  <div id="modusfield">
    <div class="actions">
      <ul>
        $OPTIONAL_EDIT
        <li>
          <a href="javascript:logbook_overview();">Zur&uuml;ck zu: Tagebuch</a>
        </li>
      </ul>
    </div>
  </div>
</div>
<table>
    <tbody>
        <tr>
            <td class="label table_header_v table_width_long">Datum</td>
            <td class="table_row_v">$DATE $TIME</td>
        </tr>
        <tr>
            <td class="label">Bearbeiter</td>
            <td class="table_row_v_2">$EDITOR</td>
        </tr>
        <tr>
            <td class="label table_header_v">Art</td>
            <td class="table_row_v">$KIND</td>
        </tr>
        <tr>
            <td class="label">Dauer in Minuten</td>
            <td class="table_row_v_2">$DURATION</td>
        </tr>
        <tr>
            <td class="label table_header_v">Notiz</td>
            <td class="table_row_v">$NOTICE</td>
        </tr>
    </tbody>
</table>
''')

NEW_ENTRY_TEMPLATE = Template(u'''
<div id="content_header_small">
  <div id="content_ueberschrift">
     Tagebucheintrag vornehmen
  </div>
  <div id="modusfield">
    <div class="actions">
      <ul>
        <li><a href="javascript:logbook_overview();">Zur&uuml;ck zu: Tagebuch</a></li>
      </ul>
    </div>
  </div>
</div>
$ENTRY_ERRORS
<div id="appointment">
<div id="waska_form">
  <form action="javascript:create_logbook_entry_store();" accept-charset="UTF-8" method="POST">
  $ENTRY_BODY
  </form>
</div>
</div>
''')

EDIT_ENTRY_TEMPLATE = Template(u'''
<div id="content_header_small">
  <div id="content_ueberschrift">
     Tagebucheintrag bearbeiten
  </div>
  <div id="modusfield">
    <div class="actions">
      <ul>
        <li><a href="javascript:logbook_overview();">Zur&uuml;ck zu: Tagebuch</a></li>
      </ul>
    </div>
  </div>
</div>
$ENTRY_ERRORS
<div id="appointment">
<div id="waska_form">
  <form action="javascript:edit_logbook_entry_store();" accept-charset="UTF-8" method="POST">
  $ENTRY_BODY
  </form>
</div>
</div>
''')

ENTRY_EDIT_BODY_TEMPLATE = Template(u'''
<fieldset>
<legend>Zeitangaben</legend>
    <div class="w25">$DATE</div>
    <div class="w25">$TIME</div>
    <div class="w25">$DURATION</div>
</fieldset>
<fieldset>
<legend>Eintrag</legend>
    <div class="w100">
        <label for="editor">Bearbeiter:</label><br>
        <input type="text" class="field"
         name="editor" id="editor" maxlength="40" value="$EDITOR">
    </div>
    <div class="w100">
        <label for="kind">Art des Eintrags:</label><br>
        <select name='kind' id='kind'>
        $KIND
        </select>
    </div>
    <div class="w100">
        <label for="short_notice">Nennung Sonstiges:</label><br>
        <input type="text" class="field"
         name="short_notice" id="short_notice" maxlength="80"
         value="$SHORT_NOTICE">
    </div>
<div>
    <label for="notice">Notiz:</label><br>
    <textarea cols="70" rows="10" name="notice" id="notice">$NOTICE</textarea>
</div>
<div>
    <input type="submit" value="Eintrag speichern">
    <input type="hidden" name="uuid" value="$UUID">
</div>
</fieldset>
''')


class LogbookEntryRenderer(object):

    def __init__(self):
        pass

    def render(self, entry):
        uuid       = entry.getUUID()
        date       = escape(entry.getDate())
        time       = escape(entry.getTime())
        editor     = escape(entry.getEditor())
        entry_type = entry.getType()
        kind       = entry_type.getDescription() if entry_type else u'-/-'
        kind       = escape(kind)
        duration   = escape(entry.getDuration())
        notice     = escape(entry.getNotice()).replace("\n", "<br>")

        if not entry_type or entry_type.isEditable():
            optional_edit = (u'<li>'
                             u'<a href="javascript:edit_logbook_entry(\'%s\');"'
                             u'>In Bearbeitung nehmen</a>'
                             u'</li>' % uuid)
        else:
            optional_edit = u''

        return VIEW_ENTRY_TEMPLATE.safe_substitute({
            'UUID'         : uuid,
            'DATE'         : date,
            'TIME'         : time,
            'EDITOR'       : editor,
            'KIND'         : kind,
            'DURATION'     : duration,
            'NOTICE'       : notice,
            'OPTIONAL_EDIT': optional_edit
            })


class LogbookOverviewRenderer(object):

    def __init__(self):
        pass

    def render(self, logbook, file_cache):
        if logbook.entries:
            sorted_entries = logbook.sorted_entries(ORDER_ENTRIES_BY_DATE_DESC)
            view_img = file_cache.get_image("images/icons/view_red.gif",
                                            'border="0" alt="Eintrag ansehen"'
                                            ' title="Eintrag ansehen"')
            delete_img = file_cache.get_image("images/icons/delete_red.gif",
                                              'border="0"'
                                              ' alt="Eintrag l&ouml;schen"'
                                              ' title="Eintrag l&ouml;schen"')
            entries = []
            for entry in sorted_entries:
                entry_type = entry.getType()
                kind = entry_type.getDescription() if entry_type else u"-/-"
                uuid     = entry.getUUID()
                date     = escape(entry.getDate())
                time     = escape(entry.getTime())
                editor   = escape(entry.getEditor())
                kind     = escape(kind)
                duration = escape(entry.getDuration())
                notice   = escape(shorten(entry.getNotice(), 120))

                if not entry_type or entry_type.isEditable():
                    optional_delete = \
                        u'<a href="javascript:delete_logbook_entry(\'%s\');"' \
                        u'>%s</a>' % (uuid, delete_img)
                else:
                    optional_delete = u''

                entries.append(ENTRY_TEMPLATE.safe_substitute({
                    'UUID'           : uuid,
                    'DATE'           : date,
                    'TIME'           : time,
                    'EDITOR'         : editor,
                    'KIND'           : kind,
                    'DURATION'       : duration,
                    'VIEW_IMAGE'     : view_img,
                    'OPTIONAL_DELETE': optional_delete,
                    'NOTICE'         : notice
                    }))
            entries = u"".join(entries)
        else:
            entries = NO_ENTRIES

        return OVERVIEW_TEMPLATE.safe_substitute({
            'OVERVIEW_ENTRIES': entries
            })

def buildFormErrors(errors, labels):
    if not errors or not errors.hasErrors():
        return u''

    items = [
        u'<div class="form_errors">'
        u'<h1><a name="error_list">Fehlerhafte Formulareingabe</a></h1>'
        u'<ul>']

    for key, value in errors.getErrors():
        items.append(u'<li><a href="#f_%s">%s</a>: %s</li>'
                     % (key, labels.get(key, key), value))

    items.append(u'</ul></div>')

    return u"".join(items)

FORM_LABELS = {
    'date': u'Datum',
    'time': u'Uhrzeit',
    'duration': u'Dauer',
    'short_notice': u'Nennung Sonstiges'
    }


class EditEntryBodyRenderer(object):

    def __init__(self):
        pass

    def render(self, entry, errors, values=None, meta=None):
        error = errors and errors.hasError or (lambda x: False)
        value = values and (lambda x: values.get(x, (u'',))[0]) or \
            (lambda x: u'')

        if error('date'):
            date = \
            u'<label for="date" class="error_font">'\
            u'<a name="f_date">Datum:</a></label><br>' \
            u'<input type="text" class="field error_box" ' \
            u'name="date" id="date" maxlength="10" value="%s"/>' % \
                escape(value('date'), True)
        else:
            date = \
            u'<label for="date">Datum:</label><br>' \
            u'<input type="text" class="field" ' \
            u'name="date" id="date" maxlength="10" value="%s"/>' % \
                escape(entry.getDate(), True)

        if error('time'):
            time = \
            u'<label for="time" class="error_font">' \
            u'<a name="f_time">Uhrzeit:</a></label><br>' \
            u'<input type="text" class="field error_box" ' \
            u'name="time" id="time" maxlength="5" value="%s"/>' % \
                escape(value('time'), True)
        else:
            time = \
            u'<label for="time">Uhrzeit:</label><br>' \
            u'<input type="text" class="field" name="time" ' \
            u'id="time" maxlength="5" value="%s"/>' % \
                escape(entry.getTime(), True)

        if error('duration'):
            duration = \
            u'<label for="duration" class="error_font">' \
            u'<a name="f_duration">Dauer in Minuten:</a></label><br>' \
            u'<input type="text" class="field error_box" ' \
            u'name="duration" id="duration" maxlength="4" value="%s"/>' % \
                escape(value('duration'), True)
        else:
            duration = \
            u'<label for="duration">Dauer in Minuten:</label><br>' \
            u'<input type="text" class="field" ' \
            u'name="duration" id="duration" maxlength="4" value="%s"/>' % \
                escape(entry.getDuration(), True)

        editor = escape(entry.getEditor(), True)
        short_notice = escape(entry.getShortNotice(), True)
        notice = escape(entry.getNotice(), True)
        uuid = escape(entry.getUUID())

        kind = []

        if meta:
            etype = entry.getType()
            for category_name, entry_types in meta.entry_categories:
                kind.append('<optgroup label="%s">' % escape(category_name))

                for entry_type in entry_types:
                    if not entry_type.isEditable():
                        continue

                    selected = (u' selected'
                                if etype and etype.value == entry_type.value
                                else u'')

                    kind.append(u'<option value="%d"%s>%s</option>'
                                % (entry_type.value, selected,
                                   escape(entry_type.getDescription())))
                kind.append("</optgroup>")

        kind = u''.join(kind)

        return ENTRY_EDIT_BODY_TEMPLATE.safe_substitute({
            'DATE'        : date,
            'TIME'        : time,
            'DURATION'    : duration,
            'EDITOR'      : editor,
            'KIND'        : kind,
            'SHORT_NOTICE': short_notice,
            'NOTICE'      : notice,
            'UUID'        : uuid
            })


class NewLogbookEntryRenderer(object):

    def __init__(self):
        pass

    def render(self, entry, errors=None, values=None, meta=None):
        entry_errors = buildFormErrors(errors, FORM_LABELS)
        entry_body = EditEntryBodyRenderer().render(entry, errors, values, meta)

        return NEW_ENTRY_TEMPLATE.safe_substitute({'ENTRY_ERRORS': entry_errors,
                                                   'ENTRY_BODY': entry_body})


class EditLogbookEntryRenderer(object):

    def __init__(self):
        pass

    def render(self, entry, errors=None, values=None, meta=None):
        entry_errors = buildFormErrors(errors, FORM_LABELS)
        entry_body = EditEntryBodyRenderer().render(entry, errors, values, meta)

        return EDIT_ENTRY_TEMPLATE.safe_substitute({
            'ENTRY_ERRORS': entry_errors,
            'ENTRY_BODY' : entry_body
            })

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
