# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import traceback

import PyQt4.QtGui as QtGui
import PyQt4.QtCore as QtCore
import PyQt4.QtWebKit as QtWebKit

from webview import WebView


__HELP = None

def show_help(content, parent=None):
    global __HELP

    if not __HELP:
        __HELP = HelpDialog(parent)

    __HELP.set_content(content)

    __HELP.show()
    __HELP.raise_()
    __HELP.activateWindow()


class HelpDialog(QtGui.QDialog):

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setWindowTitle('Hilfe')
        self.webview = QtWebKit.QWebView(self)
        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(self.webview)
        self.setLayout(vbox)
        self.resize(400, 200)

    def set_content(self, content):
        self.webview.setHtml(content)

    @QtCore.pyqtSignature("QCloseEvent")
    def closeEvent(self, evt):
        global __HELP
        __HELP = None
        QtGui.QDialog.closeEvent(self, evt)

def description(formed, name):
    c = formed.findByName(name)
    out = []
    while c:
        desc = c.getDescription()
        if desc:
            out.insert(0, desc)
        c = c.parent
    return out


class DefaultsDialog(QtGui.QDialog):

    def __init__(self, formed, defaults, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setWindowTitle('Vorgabewerte')

        self.defaults = defaults
        table = self.build_table(formed, defaults)

        take_it = QtGui.QPushButton(u"Übernehmen")
        ok = QtGui.QPushButton(u"OK")
        cancel = QtGui.QPushButton(u"Abbruch")

        hbox = QtGui.QHBoxLayout()
        hbox.addStretch(1)
        hbox.addWidget(take_it)
        hbox.addWidget(ok)
        hbox.addWidget(cancel)

        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(table)
        vbox.addLayout(hbox)

        self.setLayout(vbox)
        self.resize(200, 300)

        self.connect(take_it, QtCore.SIGNAL('clicked()'),
                     QtCore.SLOT('take_it_pressed()'))

        self.connect(ok, QtCore.SIGNAL('clicked()'),
                     QtCore.SLOT('ok_pressed()'))

        self.connect(cancel, QtCore.SIGNAL('clicked()'),
                     QtCore.SLOT('cancel_pressed()'))

    def build_table(self, formed, defaults):
        table = QtGui.QTableView(self)
        model = QtGui.QStandardItemModel()

        headers = QtCore.QStringList()
        headers.append(u"Beschreibung")
        headers.append(u"Wert")
        model.setHorizontalHeaderLabels(headers)

        for item in defaults.values:
            d = description(formed, item.key)
            if d:
                if item.description:
                    text = item.description
                else:
                    text = d[-1]
                tooltip = u" / ".join(d)
            else:
                text = tooltip = u"unbekannt"

            desc = QtGui.QStandardItem(text)
            desc.setEditable(False)
            desc.setToolTip(tooltip)
            value = QtGui.QStandardItem(item.value)
            if not item.immutable:
                value.setEditable(True)
                value.setData(QtCore.QVariant(item.value), QtCore.Qt.EditRole)
            else:
                value.setEditable(False)
            model.appendRow([desc, value])

        self.model = model
        table.setModel(model)
        table.setSortingEnabled(False)
        return table

    @QtCore.pyqtSignature('void')
    def take_it_pressed(self):
        model = self.model
        for row, item in enumerate(self.defaults.values):
            if not item.immutable:
                value = unicode(model.item(row, 1).data(QtCore.Qt.EditRole).toString())
                item.value = value
        try:
            self.defaults.store()
        except:
             QtGui.QMessageBox.critical(self, u"Fehler beim Speichern",
                                        u"Beim Speichern der Feldvorgaben"
                                        u" ist ein Fehler aufgetreten:\n%s"
                                        % traceback.format_exc())

    @QtCore.pyqtSignature('void')
    def ok_pressed(self):
        self.take_it_pressed()
        self.done(QtGui.QMessageBox.Ok)

    @QtCore.pyqtSignature('void')
    def cancel_pressed(self):
        self.done(QtGui.QMessageBox.Cancel)


class AboutDialog(QtGui.QDialog):

    def __init__(self, parent, file_cache):
        QtGui.QDialog.__init__(self, parent)
        self.setWindowTitle(u'Über den Offline-Klienten')

        ok = QtGui.QPushButton(u"OK")

        web = WebView(self, file_cache)
        web.setUrl(QtCore.QUrl("local:/about.html"))

        hbox = QtGui.QHBoxLayout()
        hbox.addStretch(1)
        hbox.addWidget(ok)

        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(web)
        vbox.addLayout(hbox)

        self.setLayout(vbox)
        self.resize(700, 450)

        self.connect(ok, QtCore.SIGNAL('clicked()'),
                     QtCore.SLOT('ok_pressed()'))

    @QtCore.pyqtSignature('void')
    def ok_pressed(self):
        self.done(QtGui.QMessageBox.Ok)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
