# -*- coding: utf-8 -*-
#
# (c) 2011 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Bernhard Herzog <bh@intevation.de>

import logging

import PyQt4.QtCore as QtCore
import PyQt4.QtWebKit as QtWebKit
import PyQt4.QtNetwork as QtNetwork


log = logging.getLogger(__name__)


class LocalReply(QtNetwork.QNetworkReply):

    """Version of QNetworkReply for local:-URLs in LocalNetworkAccessManager"""

    def __init__(self, parent, url, content):
        QtNetwork.QNetworkReply.__init__(self, parent)
        self.content = content
        self.offset = 0

        QtCore.QTimer.singleShot(0, self, QtCore.SIGNAL("readyRead()"))
        QtCore.QTimer.singleShot(0, self, QtCore.SIGNAL("finished()"))
        self.open(self.ReadOnly | self.Unbuffered)
        self.setUrl(url)

    def abort(self):
        pass

    def bytesAvailable(self):
        return len(self.content) - self.offset

    def isSequential(self):
        return True

    def readData(self, maxSize):
        if self.offset < len(self.content):
            end = min(self.offset + maxSize, len(self.content))
            data = self.content[self.offset:end]
            self.offset = end
            return data



class LocalNetworkAccessManager(QtNetwork.QNetworkAccessManager):

    """Extend QNetworkAccessManager to handle local: URLs.

    URLs beginning with local are interpreted as relative filenames that
    will be searched using a FileCache instance.
    """

    def __init__(self, file_cache):
        self.file_cache = file_cache
        QtNetwork.QNetworkAccessManager.__init__(self)

    def createRequest(self, operation, request, data):
        url = request.url()
        if url.scheme() == "local":
            path = unicode(url.path()).lstrip("/")
            try:
                contents = self.file_cache.get_file(path)
            except IOError:
                log.warning("Could not find resource for %r", path)
                contents = ""
            return LocalReply(self, url, contents)

        return QtNetwork.QNetworkAccessManager.createRequest(self, operation,
                                                             request, data)


class WebView(QtWebKit.QWebView):

    """Version of QWebView that uses a LocalNetworkAccessManager"""

    def  __init__(self, parent, file_cache):
        QtWebKit.QWebView.__init__(self, parent)
        self.networkmanager = LocalNetworkAccessManager(file_cache)
        self.page().setNetworkAccessManager(self.networkmanager)
